@@ -1,5 +1,5 @@
 package Selenium::Firefox;
-$Selenium::Firefox::VERSION = '0.26';
+$Selenium::Firefox::VERSION = '0.27';
 # ABSTRACT: Use FirefoxDriver without a Selenium server
 use Moo;
 use Selenium::CanStartBinary::FindBinary qw/coerce_firefox_binary/;
@@ -36,6 +36,7 @@ has '_binary_args' => (
 
 with 'Selenium::CanStartBinary';
 
+
 1;
 
 __END__
@@ -50,7 +51,7 @@ Selenium::Firefox - Use FirefoxDriver without a Selenium server
 
 =head1 VERSION
 
-version 0.26
+version 0.27
 
 =head1 SYNOPSIS
 
@@ -96,6 +97,17 @@ See L<Selenium::CanStartBinary/port> for more details, and
 L<Selenium::Remote::Driver/port> after instantiation to see what the
 actual port turned out to be.
 
+=head2 startup_timeout
+
+Optional: specify how long to wait for the binary to start itself and
+listen on its port. The default duration is arbitrarily 10 seconds. It
+accepts an integer number of seconds to wait: the following will wait
+up to 20 seconds:
+
+    Selenium::Firefox->new( startup_timeout => 20 );
+
+See L<Selenium::CanStartBinary/startup_timeout> for more information.
+
 =head1 SEE ALSO
 
 Please see those modules/websites for more information related to this module.
